package com.tfg.mwc14.add;

import java.io.FileNotFoundException;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.actionbarsherlock.app.SherlockActivity;
import com.actionbarsherlock.view.Menu;
import com.actionbarsherlock.view.MenuItem;
import com.tfg.mwc14.R;
import com.tfg.mwc14.common.Server;

public class AddActivity extends SherlockActivity {

	private Context context;
	// error code for the form
	private int allInfo; 
	// string where the extra answers dynamically added will be stored and finally added to the parameters
	private String extraAnswers;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState); 
		setContentView(R.layout.add_activity);
		setTitle(R.string.add_title);
		getSupportActionBar().setDisplayHomeAsUpEnabled(true);
		context = this; 
		allInfo = 0;		
		extraAnswers = "";
	
		// button to launch the image selection
		Button addButton = (Button) findViewById(R.id.select_image);
		addButton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View view) {
				Intent photoPickerIntent = new Intent(Intent.ACTION_PICK);
				photoPickerIntent.setType("image/*");
				startActivityForResult(photoPickerIntent, 4);
			}
		});

		// this button will show a dialog to add a extra answer
		Button addAnswer = (Button) findViewById(R.id.addAnswer);
		addAnswer.setOnClickListener(new View.OnClickListener() {
			public void onClick(View view) {
				getNewAnswer();				
			}
		});
	}

	// shows a input dialog and returns the string                                
	private void getNewAnswer() {		
		// text field
		final EditText textField = new EditText(this);		
		textField.setHint("Write the possible answer and press Ok");				
		// alert dialog with the text field inside
		AlertDialog.Builder dialog = new AlertDialog.Builder(this);
		dialog.setTitle("Extra answer");		
		dialog.setView(textField);
		// keyboard 
		final InputMethodManager keyboard = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
		// OK
		dialog.setPositiveButton("Ok", new DialogInterface.OnClickListener() {
		    public void onClick(DialogInterface dialog, int whichButton) {
		    	String extraAnswer = textField.getText().toString();
		    	if (!extraAnswer.equals(null) && !extraAnswer.equals("")) {
		    		// changes the blanks of extraAnswer to '*' and adds it to the string that will be sent as parameter
					extraAnswers = extraAnswers + "|" + deleteBlanks(extraAnswer);
					// gets the value in screen for the extra answers counter and updates it
					int extraCount = Integer.parseInt(((TextView) findViewById(R.id.extraCount)).getText().toString());	
					extraCount++;					
					((TextView) findViewById(R.id.extraCount)).setText(extraCount+"");		
					Toast.makeText(context, "Answer added", Toast.LENGTH_SHORT).show();
					// adds the new answer to the slider				
/*	
					LinearLayout extraAnswers = ((LinearLayout) findViewById(R.id.extraAnswersLabels));
					TextView answer = new TextView(context); 
				    answer.setText(extraAnswer);
				    // android:textSize="15sp"
			        answer.setTextSize(20); 
			        // android:layout_width="wrap_content" android:layout_height="wrap_content"    
			        LinearLayout.LayoutParams llp = new LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
			        // android:layout_marginTop="10dp"   (left, top, right, bottom)
			        llp.setMargins(0, 15, 0, 0); 
			        answer.setLayoutParams(llp);
			        extraAnswers.addView(answer);   
*/     	
		    	}  
		    	// close keyboard
		    	keyboard.hideSoftInputFromWindow(textField.getWindowToken(), 0);
		    	
		    }
		});
		// cancel
		dialog.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {
				// close keyboard
				keyboard.hideSoftInputFromWindow(textField.getWindowToken(), 0);				
		    }
		});		
		// gives focus to the text field, shows the keyboard and the dialog
		textField.requestFocus();		
		keyboard.toggleSoftInput(InputMethodManager.SHOW_FORCED, InputMethodManager.HIDE_IMPLICIT_ONLY); 
		dialog.show();
	}			
 
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent imageReturnedIntent) {
		super.onActivityResult(requestCode, resultCode, imageReturnedIntent);
		try {
			// opens a file selector (restricted to the gallery)
			if (requestCode == 4 && resultCode == RESULT_OK) {
				// we will make two copies of the image. A small preview and the original one (which will be sent)
				Uri selectedImage = imageReturnedIntent.getData();
				Bitmap image = BitmapFactory.decodeStream(getContentResolver().openInputStream(selectedImage)); // !!!!!!!!!!!!!! subir a algun server
				Bitmap preview = decodeUri(selectedImage);

				// makes the small image visible in the screen
				ImageView imageView = (ImageView) findViewById(R.id.preview);
				imageView.setImageBitmap(preview);
			}
		} catch (FileNotFoundException e) {
			Log.e("image", "cant find the pic " + e.toString());
		}
	}

	@Override
	public boolean onOptionsItemSelected(final MenuItem item) {
		switch (item.getItemId()) {
			case android.R.id.home:
				finish();
				return false;
			case R.id.menu_send:
				new UploadNewEntry().execute();
				return false;
			default:
				return super.onOptionsItemSelected(item);
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getSupportMenuInflater().inflate(R.menu.main_menu_add, menu);
		return true;
	}

	// Tries to upload a new entry to the server. AsyncTask
	private class UploadNewEntry extends AsyncTask<Void, Void, Void> {

		private Toast uploading;
		
		@Override
		protected void onPreExecute() {
			super.onPreExecute();
			uploading = Toast.makeText(context, "Uploading...", Toast.LENGTH_SHORT);
			uploading.show();
		}

		@Override
        protected Void doInBackground(Void... params) {          	
        	String idM = Server.id;

        	// gets the string in the answer1 box and checks that it's not empty. Then changes the blanks to '*'        	
        	String ans1 = ((EditText) findViewById(R.id.answer1)).getText().toString();
        	if (ans1.equalsIgnoreCase("") || ans1.equalsIgnoreCase(null)) 
        		allInfo = 3;
        	else 
        		ans1 = deleteBlanks(ans1);        		        
        	
        	// gets the string in the answer2 box and checks that it's not empty. Then changes the blanks to '*'        	
        	String ans2 = ((EditText) findViewById(R.id.answer2)).getText().toString();
        	if (ans2.equalsIgnoreCase("") || ans2.equalsIgnoreCase(null)) 
        		allInfo = 3;
        	else 
        		ans2 = deleteBlanks(ans2);
        	        	
        	// gets the string in the description box and checks that it's not empty. Then changes the blanks to '*'
        	String dsc = ((EditText) findViewById(R.id.descrition)).getText().toString();
        	if (dsc.equalsIgnoreCase("") || dsc.equalsIgnoreCase(null)) 
        		allInfo = 1;
        	else 
        		dsc = deleteBlanks(dsc);        		        	
        	
        	// gets the string in the title box and checks that it's not empty. Then changes the blanks to '*'        	
        	String tit = ((EditText) findViewById(R.id.title)).getText().toString();
        	if (tit.equalsIgnoreCase("") || tit.equalsIgnoreCase(null)) 
        		allInfo = 2;        	
        	else
        		tit = deleteBlanks(tit);
        	
			String data = "idMedico=" + idM + "&descripcion=" + dsc + "&titulo=" + tit + "&respuestas=" + ans1 + "|" + ans2 + extraAnswers + "|";			
			
			// if we have all the info, we can upload to the server 
			if (allInfo == 0)
				Server.SendToServer("http://" + Server.ip + ":1289/xampp/TFGETASTERISCO/guardar_consulta.php", data, false);        
				
            return null;
        }
 
		@Override
		protected void onPostExecute(Void args) {
			// show an error message if something happened at server side or with the connection
			String error = Server.getErrorMessage();
			if (!error.equalsIgnoreCase("ok")) 
				Toast.makeText(context, error, Toast.LENGTH_SHORT).show();
			// errors in the content of the question
			else if (allInfo == 1)
				Toast.makeText(context, "You are missing the description!", Toast.LENGTH_SHORT).show();
			else if (allInfo == 2)
				Toast.makeText(context, "You are missing the title!", Toast.LENGTH_SHORT).show();
			else if (allInfo == 3)
				Toast.makeText(context, "You need two a minimum of two answers!", Toast.LENGTH_SHORT).show();
			// process OK
			else {
				uploading.cancel();
				Toast.makeText(context, "Done!", Toast.LENGTH_SHORT).show();
				setResult(Activity.RESULT_OK);
				finish();
			}
			allInfo = 0;
		}
	}
	
	// changes blanks (' ') to '*' and deletes blanks at the beginin or end of the string
	private String deleteBlanks(String argWithBlanks) {
		argWithBlanks = argWithBlanks.trim();
		String argWithOutBlanks = "";
		char charAux;
		for (int i = 0; i < argWithBlanks.length(); i++) {
			charAux = argWithBlanks.charAt(i); 
			if (charAux == ' ')				
				charAux = '*';
			argWithOutBlanks = argWithOutBlanks + charAux;
		}		
		return argWithOutBlanks;
	}
	
	private Bitmap decodeUri(Uri selectedImage) throws FileNotFoundException {

		// Decode image size
		BitmapFactory.Options o = new BitmapFactory.Options();
		o.inJustDecodeBounds = true;
		BitmapFactory.decodeStream(getContentResolver().openInputStream(selectedImage), null, o);

		// The new size we want to scale to
		int REQUIRED_SIZE = 140;

		// Find the correct scale value. It should be the power of 2.
		int width_tmp = o.outWidth, height_tmp = o.outHeight;
		int scale = 1;
		while (true) {
			if ((width_tmp / 2 < REQUIRED_SIZE) || (height_tmp / 2 < REQUIRED_SIZE))
				break;
			width_tmp /= 2;
			height_tmp /= 2;
			scale *= 2;
		}

		// Decode with inSampleSize
		BitmapFactory.Options o2 = new BitmapFactory.Options();
		o2.inSampleSize = scale;
		return BitmapFactory.decodeStream(getContentResolver().openInputStream(selectedImage), null, o2);
	}
}
