package com.tfg.mwc14.common;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.HashMap;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.ProgressDialog;
import android.content.Context;
import android.os.AsyncTask;
import android.util.Log;
import android.widget.Toast;

import com.actionbarsherlock.app.SherlockFragment;
 
public abstract class BaseFragment extends SherlockFragment {
		
	private ProgressDialog mProgressDialog;		

	/**
	 * Context of the activity. Set it when doing onCreateView
	 */
	protected Context context;

	/**
	 * This is the table with the contents of the fragment. It's created empty, so call:
	 * 1. getDataFromCache and then updateFragment(false) 
	 * or
	 * 2. updateFragment(true) 
	 * And for the love of God, be careful!! 
	 */
	protected ArrayList<HashMap<String, String>> data = new ArrayList<HashMap<String, String>>();
	
	/**
	 * Set this number to one of the following.
	 * 0 indicates everything is OK.
	 * 1 indicates error parsing jArray. 
	 * 2 indicates error saving in disk.
	 */
	protected int error = 0;

    /** Retrieves the data currently on the disk.
     * This function is responsible of the integrity of the returned array (all the data is returned or an empty list)     
     * @param String with the name of the file to retrieve. Name only, the path is build automatically.
     * @return ArrayList<HashMap<String, String>> with the full list or a new empty one.
     */
 	@SuppressWarnings("unchecked")
 	protected ArrayList<HashMap<String, String>> getDataFromCache(String file) {
 		try {	
 			File dataFile = new File(android.os.Environment.getExternalStorageDirectory() + File.separator + "TFGMWC14", file); 			
 			ObjectInputStream ois = new ObjectInputStream(new BufferedInputStream(new FileInputStream(dataFile)));		
 			ArrayList<HashMap<String, String>> temp = (ArrayList<HashMap<String, String>>) ois.readObject();
 			ois.close();
 			return temp;
 		}		
 		// if any of the data couldn't be restored or if it was no data to restore, we return an empty list, so no problem here
 		catch (Exception e) {
 			Log.e("Error", "Error recurperando la lista de datos " + e.toString());			
 		} 
 		return new ArrayList<HashMap<String, String>>();
 	}

 	/**
 	 * Saves data to the disk.
 	 * The path to the file is fixed don't need to worry about that. If something fails, no data is written.
 	 * @param table what we want to save.
 	 * @param file name of the file when we want to save it. 
 	 * @throws Exception if something went wrong.
 	 */
 	protected void saveData(ArrayList<HashMap<String, String>> table, String file) throws Exception {		 		
 		File cacheDir = new File(android.os.Environment.getExternalStorageDirectory() + File.separator + "TFGMWC14");
 		File cacheFile = new File(cacheDir, file);			
 		if (!cacheDir.exists())
 			cacheDir.mkdirs();
 		ObjectOutputStream oos = new ObjectOutputStream(new BufferedOutputStream((new FileOutputStream(cacheFile))));
 		oos.writeObject(table);
 		oos.close();		
 	}

 	/** Adds the new objects to the data table. 
 	 * If it doesn't work the exception must be caught by the caller. 	 
 	 * @param jsonObject file to parse
 	 * @param data table to add new entries
 	 * @return modified data table.
 	 * @throws JSONException if parse went wrong
 	 */
  	protected ArrayList<HashMap<String, String>> parse(JSONObject jsonObject, ArrayList<HashMap<String, String>> data) throws JSONException {  		
      	// Retrieve JSON Objects
      	JSONArray jsonArray = jsonObject.getJSONArray("consultas");
         for (int i = 0; i < jsonArray.length(); i++) {
         	HashMap<String, String> map = new HashMap<String, String>();
             jsonObject = jsonArray.getJSONObject(i);
             JSONObject jsonObjectAux;
             JSONArray jsonArrayAux;
             
             // Fields of the JSON Object
             map.put("idConsulta", jsonObject.getString("idConsulta"));
             map.put("titulo", jsonObject.getString("titulo"));            
             map.put("descripcion", jsonObject.getString("consulta"));
             
             // This field is actually an array of fields             
             jsonArrayAux = jsonObject.getJSONArray("respuestas");             
             for (int j = 0; j < jsonArrayAux.length(); j++) {
            	 jsonObjectAux = jsonArrayAux.getJSONObject(j);
            	 map.put("respuesta"+j, jsonObjectAux.getString("respuesta"));
             }
             
             // Another array of fields
             jsonArrayAux = jsonObject.getJSONArray("idRespuestas");
             for (int j = 0; j < jsonArrayAux.length(); j++) {
            	 jsonObjectAux = jsonArrayAux.getJSONObject(j);
            	 map.put("idRespuesta"+j, jsonObjectAux.getString("idRespuesta"));
             }
             
             // Another array of fields
             jsonArrayAux = jsonObject.getJSONArray("contadores");
             for (int j = 0; j < jsonArrayAux.length(); j++) {
            	 jsonObjectAux = jsonArrayAux.getJSONObject(j);
            	 map.put("contador"+j, jsonObjectAux.getString("contador"));
             }           
             
             // the URL where the image is hosted
            // map.put("img", jsonObject.getString("$img"));
             map.put("img", "http://www.androidbegin.com/tutorial/flag/japan.png");
             
             // put the JSON Object into the array (at the first place). ONLY IF IT'S NOT REPEATED!!!
             if (!data.contains(map)) {
             	data.add(0, map);
              	Log.e("Info", "se ha aadido una entrada a la lista, el total es: " + data.size());
             }
             else {
             	Log.e("Info", "no se ha aadido una entrada a la lista. Era repetida, el total es: " + data.size());
             }                          
         }
		return data;         
  	}
 	
  	/**
  	 * Sends a yes/no to the server. 
  	 * It's a confirmation message. Use it when you see fit. 
  	 * @param message String with values: "yes" or "no"
  	 */
  	protected void messageToServer(String message) {
  		//Server.SendToServer("URL", message, false);							
  	}
  	
  	/**
  	 * Returns the URL to download from.
  	 */
  	protected abstract String downloadFromWhere();
  	
  	/**
	 * Use to show the 'nothing here' image.
	 * This one is obviously safe.
	 */
  	protected abstract void nothingToShow();
  	
  	/**
     * Use to show the table of contents. 
     * It's not safe. Meaning, you should have something to show at 'data' table. 
     */
  	protected abstract void showTable();
  	
  	/** Call this to start the async task that will handle the update process.
  	 *  This will not load from cache! If you want, do it yourself with getDataFromCache. 	 	
	 * @param mode true to update from server. false to show the contents of 'data'.
	 */
  	public void updateFragment(boolean mode) {  		
        // Execute Update AsyncTask
		new UpdateFragment(mode).execute();
  	}
  	  	  	
  	/**
  	 * Will be called after a successful download.
  	 * @param jsonObject the downloaded JSON ready to be parsed.
  	 */
  	protected abstract void onBackground(JSONObject jsonObject); 
  	
  	// Updates the fragment contents AsyncTask
    private class UpdateFragment extends AsyncTask<Void, Void, Void> {    	
    	
    	private boolean download;    	
    	
    	// Sets the mode of the task. True to attempt the update. False will only reload from cache    	     	
    	private UpdateFragment(boolean download) {
    		this.download = download;
    	}
    	
        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            // Shows a progress dialog only if it's a update process (will take longer)    
            if (download) {
	            // Create a progress dialog
	            mProgressDialog = new ProgressDialog(context);
	            // Set progress dialog title
	            mProgressDialog.setTitle("Updating list");
	            // Set progress dialog message
	            mProgressDialog.setMessage("Loading...");
	            mProgressDialog.setIndeterminate(false);
	            // Show progress dialog
	            mProgressDialog.show();
            }
        }

		@Override
        protected Void doInBackground(Void... params) {           
            // tries to update from server            
            if (download) {
            	String idM = Server.id;					
            	String consulta = "idMedico=" + idM;
            	JSONObject jsonObject = Server.SendToServer(downloadFromWhere(), consulta, true);
            	// do we have something new?
	            if (jsonObject != null) { 			
	            	onBackground(jsonObject);	 		            			            
	            }	            
	        }
            return null;
        }
        
        @Override
        protected void onPostExecute(Void args) {
        	
        	// shows the new list
        	if (!data.isEmpty()) 
        		showTable();        	
        	// nothing to show, shows the empty list picture
        	else 
        		nothingToShow();        	

        	// Close the progress dialog (if it was opened it the first place)
        	if (download)
        		mProgressDialog.dismiss();    
            
        	// show an error message if something went wrong on server side
        	String errorServer = Server.getErrorMessage();
            if (!errorServer.equalsIgnoreCase("ok"))           	
                Toast.makeText(context, errorServer, Toast.LENGTH_SHORT).show();                           
            // error message if something happened in client side
            else if (error != 0) {
            	String text = "";
            	if (error == 1)
            		text = "Error: parse failed";
                else if (error == 2)
                	text = "Error: cache couldn't be updated";
                Toast.makeText(context, text, Toast.LENGTH_SHORT).show();
                error = 0;
            }        
            // all worked
            else if (download)
            	Toast.makeText(context, "Updated!", Toast.LENGTH_SHORT).show();
      	}                  	        
    }
}