package com.tfg.mwc14.common;

import java.util.HashMap;

import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.actionbarsherlock.app.SherlockActivity;
import com.actionbarsherlock.view.Menu;
import com.actionbarsherlock.view.MenuItem;
import com.tfg.mwc14.R;
import com.tfg.mwc14.auxiliary.ImageLoader;

public abstract class BaseSingleItemActivity extends SherlockActivity {

	// context of the base activity, just in case that any of the 'extenders' of this class needs it
	protected Context context;	   
	// entry of the listViewAdapter who's info will be shown 
	protected HashMap<String, String> element; 
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState); 		
		// loads the base UI (title, description and preview image), sets the back button of the action bar and sets the context (remember it was protected?)
		setContentView(R.layout.base_activity);				
		getSupportActionBar().setDisplayHomeAsUpEnabled(true);
		context = this;
		ImageLoader imageLoader = new ImageLoader(context);		
		// gets the hashMap passed by listViewAdapter
		element = (HashMap<String, String>) getIntent().getSerializableExtra("element");
		// sets the title and description
		((TextView) findViewById(R.id.baseTitle)).setText(element.get("titulo"));		
		((TextView) findViewById(R.id.baseDescription)).setText(element.get("descripcion"));
		// sets the preview image (it's clickable to make it full screen)	
		ImageView baseImage = (ImageView) findViewById(R.id.baseImage);		
		imageLoader.DisplayImage(element.get("img"), baseImage);		
		// screen title
		setTitle(element.get("titulo"));
	}
	
	// triggered when the preview image is clicked. Will launch a new activity with the image in full screen 
	public void imageClick(View view) {  			
		Intent imageViewIntent = new Intent(this, ImageViewActivity.class);
		imageViewIntent.putExtra("img", element.get("img"));
		imageViewIntent.putExtra("title", element.get("titulo"));
		startActivity(imageViewIntent); 	
	} 
	
	@Override
	public boolean onOptionsItemSelected(final MenuItem item) {
		switch (item.getItemId()) {
			case android.R.id.home:
				finish();
				return false;		
			default:
				return super.onOptionsItemSelected(item);
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// empty menu (back button only)
		getSupportMenuInflater().inflate(R.menu.main_menu_base, menu);		
		return true;
	}	
	
	/**
	 * Called when the process of sending something to the server ends.
	 * @param success boolean to indicate if the process was successful or not.
	 */
	protected abstract void onProcessFinish(boolean success);
	
	/**
	 * Use this to send something to the server.
	 * @param messageS String to show before the sending process starts.
	 * @param messageE String to show when the process ends successfully.
	 * @param data String with formated parameters to send. All but idMedico, which will be auto attached here.
	 * @param dir String with the URL to send (URL only, no '?' nor parameters or anything else).
	 */
	public void sendToServerFunc(String messageS, String messageE, String data, String dir) {
		new SendToServerAsync(messageS, messageE, data, dir).execute();
	} 
	
	// Updates the fragment contents AsyncTask
    private class SendToServerAsync extends AsyncTask<Void, Void, Void> {    	
    	    	
    	private String messageS, messageE, data, dir;    	
    	private Toast startingMessage;
    	    	    	     
    	private SendToServerAsync(String messageS, String messageE, String data, String dir) {
    		this.messageS = messageS;
    		this.messageE = messageE;
    		this.data = data;
    		this.dir = dir;    		
    	}
    	
    	@Override
		protected void onPreExecute() {
			super.onPreExecute();
			startingMessage = Toast.makeText(context, messageS, Toast.LENGTH_SHORT);
			startingMessage.show();
		}    	
    	
		@Override
        protected Void doInBackground(Void... params) {           
			// tries to send to server            
			String idM = Server.id;					
			data = "idMedico=" + idM + data;
			Server.SendToServer(dir, data, false);                     
			return null;
        }
        
        @Override
        protected void onPostExecute(Void args) {        	
        	// show an error message if something went wrong on server side
        	String errorServer = Server.getErrorMessage();
        	if (!errorServer.equalsIgnoreCase("ok")) {      
        		startingMessage.cancel();
        		Toast.makeText(context, errorServer, Toast.LENGTH_SHORT).show();        		
        		onProcessFinish(false);		
        	}
        	// all worked
        	else {
        		startingMessage.cancel();
        		Toast.makeText(context, messageE, Toast.LENGTH_SHORT).show();        		
        		onProcessFinish(true);
        	}
      	}                  	        
    }
}