package com.tfg.mwc14.common;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.net.UnknownHostException;

import org.json.JSONException;
import org.json.JSONObject;

import android.util.Log;

public class Server {

	public static String id = "1";
	public static String ip = "88.7.130.34";
	
	private static JSONObject jArray = null;
	private static String result = null;
	private static int error = 0;    

	/**
	 * Connects to the specified URL, downloads and makes a jArray from it.
	 * You can read the resulting status through getErrorCode or getErrorMessage. There is a 3 seconds time out to reach the server.	
	 * @param dir URL to read from (DON'T ADD ANY PARAMETER NOR '?' OR ANYTHING TO THE URL). 
	 * @param data parameters for the URL (DON'T ADD '?' OR ANYTHING ELSE)
	 * @param json set this flag to true if you expect a json from the server. false if you expect a yes/no 
	 * @return json or null
	 */
    public static JSONObject SendToServer(String dir, String data, boolean json) {
    	Log.e("url", dir);
    	Log.e("enviando", data);
    	
    	final int TIMEOUT_VALUE = 3000;    	
    	HttpURLConnection connection = null;    			
		URL url;
		try {			
			// URL -> InputStream
		    url = new URL(dir + "?" + data);		    
		    connection = (HttpURLConnection) url.openConnection();   	
		    connection.setConnectTimeout(TIMEOUT_VALUE);		    
		    connection.setReadTimeout(TIMEOUT_VALUE);
		    connection.setDoInput(true);
		    connection.setDoOutput(true); 
		    connection.setUseCaches(false); 		              
		    connection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
		    
		    // InputStream -> Strings
		    BufferedReader br = new BufferedReader(new InputStreamReader(connection.getInputStream()));
		    StringBuilder sb = new StringBuilder();
		    String line = br.readLine();
		    while (line != null) {
		    	sb.append(line);
		    	Log.e("recivido", "leido: "+line);
		    	line = br.readLine();
		    }                  
		    br.close();
		    result = sb.toString();

		    // Strings -> jArray
		    if (json)
		    	jArray = new JSONObject(result);
		    else if (!result.equalsIgnoreCase("si")) {	
		    	error = 7;
		    	Log.e("ERROR", "error en el server " + result);		    	
		    }		    	
			
			connection.disconnect();			
		    return jArray;
		}				
		catch (Exception e) {			 
			if (e instanceof SocketTimeoutException) {
				Log.e("ERROR", "time out " + e.toString());
				error = 6;
			}
			else if (e instanceof MalformedURLException) {
				Log.e("ERROR", "Url incorrecta " + e.toString());
				error = 2;
			}
			else if (e instanceof ProtocolException) {
				Log.e("ERROR", "Error de protocolo " + e.toString());
				error = 3;	
			}			
			else if (e instanceof JSONException) {
				Log.e("ERROR", "Error parsing data " + e.toString());
				error = 4;
			}			
			else if (e instanceof UnknownHostException) {
				Log.e("ERROR", "Host desconocido " + e.toString());
				error = 3;	
			}
			else if (e instanceof FileNotFoundException) {
				Log.e("ERROR", "Servidor no encontrado " + e.toString());
				error = 3;	
			}
			else if (e instanceof IOException) {
				Log.e("ERROR", "Io error " + e.toString());
				error = 1;	
			}
			else {
				Log.e("ERROR", "Unknown error " + e.toString());
				error = 5;
			}
		}		
		return null;		 		 					       
    }        
    
    /**
     * Call this if you want the message corresponding to the last operation instead of the number.
     * The code will be reset after.
     * @return string with values:
     * OK
     * Error: check your Internet connection!
     * Error: malformed URL?
     * Error: server may be down
     * Error: parsing error
     * Error: connection time out
     * Error: something went wrong     
     * Error: something happend at server side
     */
    public static String getErrorMessage() {
    	String text = "ok";
    	int error = getErrorCode();
        if (error == 1)
        	text = "Error: check your internet connection!";
        else if (error == 2)
        	text = "Error: malformed url?";
        else if (error == 3)
        	text = "Error: server may be down";   
        else if (error == 4)
          	text = "Error: parsing error";
        else if (error == 6)
        	text = "Error: connection timed out";
        else if (error == 5)
        	text = "Error: something went wrong";
        else if (error == 7)
        	text = "Error: something happend at server side";
        return text;
    }
    
    /**
     * Returns the value of a flag indicating the result of the last operation.
     * Then the flag is reseted.
     * Values are:
	 * 0 indicates everything is OK.
	 * 1 indicates IO error.
	 * 2 indicates wrong URL.
	 * 3 indicates something wrong with protocol. 	
	 * 4 indicates error during parse. 	
	 * 5 indicates another error.
	 * 6 indicates server time out.
     */
    public static int getErrorCode() {    	
    	int aux = error;
    	error = 0;
    	return aux;    	
    }
}
