package com.tfg.mwc14.inbox;

import java.util.ArrayList;
import java.util.HashMap;

import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ListView;

import com.tfg.mwc14.R;
import com.tfg.mwc14.common.BaseFragment;
import com.tfg.mwc14.common.BaseListViewAdapter;
import com.tfg.mwc14.common.Server;

public class InBoxFragment extends BaseFragment {
 
    /**
     * Use this to force the fragment to think that is the first time it opens.
     * Leave it static for the love of god.
     */
	public static boolean isFirstTime = true;
	
	@Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.inbox_fragment, container, false);     
        context = getActivity();
        
        // tries to update from the server only if it's the first time the fragment opens        
        if (isFirstTime) {
        	isFirstTime = false;
        	updateFragment(true);
        }
        // if it's just re gaining focus, just shows the contents in cache
        else
        	updateFragment(false);
        
        return rootView;
    }
			
	// there are two options: download the whole package or just a update, depends on the data table. That's why we loaded it before.
    @Override
	protected String downloadFromWhere() {        
    	// if the local list is empty we tell the server to send all his list
    	if (data.isEmpty()) 
    		return "http://" + Server.ip + ":1289/xampp/TFGETASTERISCO/mostrar_consultas_resolver.php";	                           
    	// if the local list is not empty we tell the server to send just the update
        else  
        	return "http://" + Server.ip + ":1289/xampp/TFGETASTERISCO/mostrar_consultas_resolver.php";						  	
	}	
    
    // in this case we want to load from cache before updating the fragment, so we can ask the server just for a update and not all the package
    // the load process can't be done when onCreateView is executed, because it wouldn't do it when the update button is pressed 
    @Override
	public void updateFragment(boolean mode) {		    	
        data = getDataFromCache("data1");
        super.updateFragment(mode);
	}

	@Override
	protected void onBackground(JSONObject jsonObject) {
		// tries to parse and save the downloaded json object. If all works notify the server
        try {
        	data = parse(jsonObject, new ArrayList<HashMap<String, String>>());    		//parse(jsonObject, data);          
            saveData(data, "data1");		               
            messageToServer("yes");																		            	
        }
        // if the parsing went wrong or if the data couldn't be saved
        catch (Exception e) {
        	// let the server know that the process was not completed 
            messageToServer("no");																				            	
        	// if the parse went wrong the data table will be messed up, load something safe
        	if (e instanceof JSONException) {		            		
        		data = getDataFromCache("data1");
            	error = 1;
        		Log.e("Error", "Error parseando el json " + e.toString());		            		
        	}
        	// if save fails the data table is not written locally. No local changes are made and the server knows that the update didn't work, it will send it again next time 
        	else {
        		error = 2;
        		Log.e("Error", "Error guardando la tabla de datos " + e.toString());
        	}
        }	      	
	} 
		
    protected void nothingToShow() {
    	getView().findViewById(R.id.listview1).setVisibility(View.INVISIBLE);    
    	getView().findViewById(R.id.nothingToShow1).setVisibility(View.VISIBLE);
    }
        
    protected void showTable() {
    	// hides the empty list picture. There's something in the table so we'll show that   
		getView().findViewById(R.id.nothingToShow1).setVisibility(View.INVISIBLE);		
		// Locate the listview in inbox_fragment.xml        	
		ListView inboxList = (ListView) getView().findViewById(R.id.listview1);   
		inboxList.setVisibility(View.VISIBLE);  
        // Pass the results into ListViewAdapter.java
		BaseListViewAdapter adapter = new InBoxListViewAdapter(getActivity(), data);
        // Set the adapter to the ListView
        inboxList.setAdapter(adapter);
        // captures the click event
        inboxList.setOnItemClickListener(new OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {					
				Intent intent = new Intent(context, InBoxSingleItemActivity.class);
		        // Pass the data as a hashMap object      
		        intent.putExtra("element", data.get(position));
		        // Puts the position of the element, will be returned by the activity to be deleted of the table
		        intent.putExtra("position", position);  
		        // Start SingleItemView Class and wait for it to end
		        startActivityForResult(intent, 2);  
			}           
        });
    }  
    
    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent intent) {    	
    	if (requestCode == 2 && resultCode == Activity.RESULT_OK && intent != null) {
    		// deletes the already answered question and saves the list to disk
    		int position = intent.getIntExtra("position", 0);
	    	HashMap<String, String> element = data.get(position);
	    	data.remove(element);
	        try {
	        	saveData(data, "data1");
	        }
	        catch (Exception e) {        	
	        }
	        // finally updates the fragment from the local data
	        updateFragment(false);    	
    	}
    }
}