package com.tfg.mwc14.menu;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentTransaction;
import android.view.View;

import com.actionbarsherlock.view.Menu;
import com.actionbarsherlock.view.MenuItem;
import com.jeremyfeinstein.slidingmenu.lib.SlidingMenu;
import com.jeremyfeinstein.slidingmenu.lib.app.SlidingFragmentActivity;
import com.tfg.mwc14.R;
import com.tfg.mwc14.add.AddActivity;
import com.tfg.mwc14.historic.HistoricFragment;
import com.tfg.mwc14.inbox.InBoxFragment;
import com.tfg.mwc14.inprogress.InProgressFragment;
import com.tfg.mwc14.options.OptionsFragment;

public class MainActivity extends SlidingFragmentActivity {

    private Fragment inBox = new InBoxFragment();
    private Fragment inProgress = new InProgressFragment();
    private Fragment historic = new HistoricFragment();
    private Fragment options = new OptionsFragment();
    private boolean returningWithResult = false;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setContentView(R.layout.responsive_content_frame);
		setTitle(R.string.inbox_title);		

		// check if the content frame contains the menu frame (single fragment)
		if (findViewById(R.id.menu_frame) == null) {
			setBehindContentView(R.layout.side_menu_frame);
			getSlidingMenu().setSlidingEnabled(true);
			getSlidingMenu().setTouchModeAbove(SlidingMenu.TOUCHMODE_FULLSCREEN);
			// show home as up so we can toggle
			getSupportActionBar().setDisplayHomeAsUpEnabled(true);
		} 
		// (two fragments)
		else { 
			// add a dummy view			
			setBehindContentView(new View(this)); 
			getSlidingMenu().setSlidingEnabled(false);
			getSlidingMenu().setTouchModeAbove(SlidingMenu.TOUCHMODE_NONE);
		}
  
		// set the the main Fragment
		if (savedInstanceState == null)  			
			 getSupportFragmentManager().beginTransaction().replace(R.id.content_frame, inBox).commit();

		// set the menu Fragment
		getSupportFragmentManager().beginTransaction().replace(R.id.menu_frame, new MenuFragment()).commit();

		// customize the SlidingMenu
		SlidingMenu sm = getSlidingMenu();
		sm.setBehindOffsetRes(R.dimen.slidingmenu_offset);
		sm.setShadowWidthRes(R.dimen.shadow_width);
		sm.setShadowDrawable(R.drawable.shadow);
		sm.setBehindScrollScale(0.25f);
		sm.setFadeDegree(0.25f);
		this.setSlidingActionBarEnabled(false);
  
	}	

	/**
	 * Switches between fragments	 
	 * @param pos position of the item selected in slidingMenu
	 */
	public void switchContent(int pos) {		
		 FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
		if (pos == 0 && !inBox.isVisible()) {
			ft.replace(R.id.content_frame, inBox).commit();
			setTitle(R.string.inbox_title);
		}
		else if (pos == 1 && !inProgress.isVisible()) {
			ft.replace(R.id.content_frame, inProgress).commit();
			setTitle(R.string.inprogress_title);
		}		
		else if (pos == 2 && !historic.isVisible()) {
			ft.replace(R.id.content_frame, historic).commit();
			setTitle(R.string.historic_title);
		}
		else if (pos == 3 && !options.isVisible()) {
			ft.replace(R.id.content_frame, options).commit();			
			setTitle(R.string.options_title);
		}
		
		new Handler().postDelayed(new Runnable() {
			public void run() {
				getSlidingMenu().showContent();
			}
		}, 50);		
	}	

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getSupportMenuInflater().inflate(R.menu.main_menu, menu);
		return true; 
	}
	 
	@Override
	public boolean onOptionsItemSelected(final MenuItem item) {
		switch (item.getItemId()) {
			// shows the side menu
			case android.R.id.home:
				setTitle(R.string.main_title);
				toggle();		
				return false;				 
			// tries to refresh the fragment that is visible
	        case R.id.menu_refresh:
	        	// switch to a progress animation
	        	item.setActionView(R.drawable.indeterminate_progress_action);	        	
	        	// updates the fragment that is visible
		        if (inBox.isVisible())                         	
		        	((InBoxFragment) inBox).updateFragment(true);  	                   
		        else if (inProgress.isVisible())                         	
		        	((InProgressFragment) inProgress).updateFragment(true);		        
		        else if (historic.isVisible())                         	
		        	((HistoricFragment) historic).updateFragment(true);		        
                // stops the spin animation after one second
		        new Handler().postDelayed(new Runnable() {
	        		   public void run() { 
		                item.setActionView(null); 
	        		   }
		        }, 1000);
		        return false;		        
	        // shows the activity to create a new entry
	        case R.id.menu_new: 	        	
	        	Intent addIntent = new Intent(MainActivity.this, AddActivity.class);	
	        	// waits for the activity to finish so it can update for server (to show the newly created entry)
				startActivityForResult(addIntent, 1); 				
	            return false;	 	           
	        default:
	            return super.onOptionsItemSelected(item);
		}		
	}	 
	
	@Override
    public void onActivityResult(int requestCode, int resultCode, Intent intent) {
		super.onActivityResult(requestCode, resultCode, intent);
	    if (requestCode == 1 && resultCode == Activity.RESULT_OK && intent == null)
	    	returningWithResult = true;		
    }
	
	@Override
	protected void onPostResume() {
	    super.onPostResume();
	    // if the add activity finished and a new entry was added, this will take to the in progress fragment and update from server (so you can see the new one)
    	if (returningWithResult) {
    		// I do the update from server by setting 'first time' instead of using 'updateFragment(true)' 
    		// thats because the process of replacing the fragment is ASYNK so if I cannot do it and 'updateFragment(true)' immediately after (fragment's onCreateView is not yet completed)
    		InProgressFragment.isFirstTime = true;
    		getSupportFragmentManager().beginTransaction().replace(R.id.content_frame, inProgress).commit();
    		setTitle(R.string.inprogress_title);    		  	    	
    	}
	    // Reset the boolean flag back to false for next time.
	    returningWithResult = false;
	}
}
